<?php
    /**
    *
    * copyright (c) 2013-2019 Heliac Technologies
    * license http://opensource.org/licenses/gpl-license.php GNU GPL v3 
    *
    * linee guida per l'utilizzo del logo http://www.geecom.org/articolo.php?id=34&alias=linee-guida-per-l%27uso-del-logo-geecom
    *
    * Per ottimizzare il caricamento delle pagine al momento del salvataggio di un componente
    * viene generato il suo codice, che poi verrà caricato nel campo testo della tabella 
    * geecAd_composizionePagine. In questo modo si risparmiano le query e il tempo di generazione
    * di quella pagina sarà molto più basso.
    * Non tutti i componenti potranno essere precaricati, il blog e gli articoli dal blog ad 
    * esempio non potrà esserlo.
    * 
    * Lo schema di funzionamento sarà il seguente
    *  
    *  1) al momento di salvare un componente, dopo aver salvato i nuovi dati Geecom invocato
    *     il generatore di codice del componente, che produrrà il codice che quell'occorrenza
    *     del componente deve mostrare
    *
    *  2) successivamente Geecom cercherà nella tabella geecAd_composizionePagine tutte le occorrenze 
    *     che mostrano quel componente (accoppiata idComponente più idElemento) e le aggiornerà con 
    *     il nuovo codice
    * 
    *  3) quando una pagina viene richiamata, i componenti precaricati non dovranno essere generati,
    *     verrà semplicemente stampato il codice memorizzato in essi.
    *
    * La classe GeneratoreCodiceComponente fornisce i metodi base che accomunano tutti i generatori
    * con i metodi 
    */
    class GeneratoreCodiceComponente {
        private $db;
        private $idComponente;
        private $codiceHtml;
        private $script;
        
        function __construct($db) {
            $this->db = $db;
            $this->codiceHtml = "";
            $this->script = "";
            $datiComponente = $this->db->leggiSingolaRiga("geecAd_componenti","Risorse","nome");
            $this->idComponente = $datiComponente["id"];
        }
        
        /*
         * Questo metodo cerca tutte le occorrenze del componente che si è appena aggiornato
         * nella tabella geecAd_composizionePagine. Per ogni occorrenza prende i dati utili (es
         * la classe della sezione e l'id, che potrebbe essere diverso ogni volta che è stato
         * inserito ) e poi invoca i metodi generaCodiceHtml e generaCodiceJavascript passando
         * questi parametri. DOpo aver generato il codice aggiorna quella occorrenza.
         *
         * Questo viene ripetuto per ogni occorrenza trovata
         * 
         * Prende in input l'id dell'occorrenza del componente
         */
        function aggiornaSingolaIstanza($id) {
            $queryRicercaComponenti = "SELECT * FROM geecAd_composizionePagine WHERE idComponente=$this->idComponente AND idElemento=$id";
            $risComponenti = $this->db->eseguiQueryLettura($queryRicercaComponenti);
            
            while($datiComponente = $risComponenti->fetch_assoc()) {
            echo "pio<br>";
                $this->generaCodiceHtml($datiComponente);
                $this->generaCodiceJavascript($datiComponente);
                
                $queryAggiornamento = "UPDATE geecAd_composizionePagine SET testo = '$this->codiceHtml', script='$this->script' "
                        . "WHERE idComponente=$this->idComponente AND idElemento={$datiComponente["idElemento"]}";
                $this->db->eseguiQueryLettura($queryAggiornamento);
            }
            
        }
        
        /*
         * Questo metodo aggiorna tutte le istanze del componente. Non prende in input un campo id
         * perché tramite il parametro idComponente recupera tutte le occorrenze e le aggiorna una
         * per una
         */
        function aggiornaTutteLeIstanze() {
            $queryRicercaComponenti = "SELECT * FROM geecAd_composizionePagine WHERE idComponente=$this->idComponente";
            $risComponenti = $this->db->eseguiQueryLettura($queryRicercaComponenti);
            
            while($datiComponente = $risComponenti->fetch_assoc()) {
                $this->generaCodiceHtml($datiComponente);
                $this->generaCodiceJavascript($datiComponente);
                
                $queryAggiornamento = "UPDATE geecAd_composizionePagine SET testo = '$this->codiceHtml', script='$this->script' "
                        . "WHERE idComponente=$this->idComponente AND idElemento={$datiComponente["idElemento"]}";
                $this->db->eseguiQueryLettura($queryAggiornamento);
            }
            
        }
        
        /*
         * questo metodo va sovrascritto dalla sottoclasse con il codice
         * opportuno per il componente specifico che si vuole stampare
         */
        function generaCodiceHtml($id) {
            
        }

        /*
         * questo metodo va sovrascritto dalla sottoclasse con il codice
         * opportuno per il componente specifico che si vuole stampare
         */
        function generaCodiceJavascript($id) {
            
        }
        
        // getter e setter
        function setDb($db) {
            $this->db = $db;
        }

        function getDb() {
            return $this->db;
        }
        
        function getCodiceHtml() {
            return $this->codiceHtml;
        }

        function getScript() {
            return $this->script;
        }

        function setCodiceHtml($codiceHtml) {
            $this->codiceHtml = $codiceHtml;
        }

        function setScript($script) {
            $this->script = $script;
        }
    }