<?php
    /**
    *
    * copyright (c) 2015-2018 Heliac Technologies
    * license http://opensource.org/licenses/gpl-license.php GNU GPL v3 
    *
    */

    class DbManager {
        public $db;
        public $nome;
        public $host;
        public $utente;
        public $password;
        public $debug;

        public function __construct($costantiDatabase) {
            $database = new mysqli($costantiDatabase["host"], $costantiDatabase["utenteDb"], $costantiDatabase["passwordDb"], $costantiDatabase["nomeDb"]);

            if (mysqli_connect_errno()) {
                echo "Errore durante la connessione al Database: ".mysqli_connect_error();
                exit();
            }
            $this->db = $database;
            $this->db->set_charset($costantiDatabase["codifica"]);
            $this->nome = $costantiDatabase["nomeDb"];
            $this->host = $costantiDatabase["host"];
            $this->utente = $costantiDatabase["utenteDb"];
            $this->password = $costantiDatabase["passwordDb"];
            $this->debug = $costantiDatabase["debug"];
        }

        /*
         * metodo che consente di eseguire l'escape string di una stringa
        *  @param la $stringa su cui eseguire l'escape
        */
        public function escapeString($stringa) {
            return $this->db->real_escape_string($stringa);
        }

        /*
         * metodo che consente di eseguire una query in lettura
        *  @param la $query la query da eseguire
        */
        public function eseguiQueryLettura($query) {
            if(!$risQuery = $this->db->query($query)) {
                if($this->debug=="si") {	//stampa l'errore solo se questi sono abilitati		
                    echo "Errore durante l'esecuzione della query: ".$this->db->error;
                    echo "<br>$query";
                }
                exit();
            }
            else {
                return $risQuery;
            }
        }

        /*
         * metodo per eseguire una query in cui viene creato un elemento. 
         * Restituisce l'id dell'elemento creato
         * @param la $query la query da eseguire
        */
        public function eseguiQueryCreazione($query) {
            if(!$risQuery = $this->db->query($query)) {
                if($this->debug=="si") {	//stampa l'errore solo se questi sono abilitati		
                    echo "Errore durante l'esecuzione della query: ".$this->db->error;
                    echo "<br>$query";
                }
                exit();
            }
            else {
                return $this->idUltimoElemento();
            }
        }

        /*
         * metodo che consente di eseguire una query in scrittura
         * @param la $query la query da eseguire
         */
        public function eseguiQueryScrittura($query) {
            if(!$risQuery = $this->db->query($query)) {
                if($this->debug=="si") {	//stampa l'errore solo se questi sono abilitati		
                    echo "Errore durante l'esecuzione della query: ".$this->db->error;
                    echo "<br>$query";
                }
                exit();
            }
        }
        
        /*
         * metodo che consente di eseguire la cancellazione di un elemento dalla tabella
         * passata in input. La query viene costruita con i parametri passati in input
         * @param $tabella la tabella sulla quale eseguire la query
         * @param $id l'id dell'oggetto da eliminare
         */
        public function eseguiQueryCancellazione($tabella,$id) {
            $query = "DELETE FROM $tabella WHERE id=$id";
            if(!$risQuery = $this->db->query($query)) {
                if($this->debug=="si") {	//stampa l'errore solo se questi sono abilitati		
                    echo "Errore durante l'esecuzione della query: ".$this->db->error;
                    echo "<br>$query";
                }
                exit();
            }
        }
        
        /*
         * legge una singola riga del database, restituendo l'array associativo contenente 
         * tutti i suoi elementi, false se non trova l'elemento corrispondente
         * 
         * @param $tabella la tabella sulla quale eseguire la query
         * @param $valoreDaCercare contiene il valore della colonna da cercare
         * @param $colonnaDaCercare contiene il nome della colonna da cercare. È un campo opzionale,
         * se non viene inserito usa il valore di default che è id
         * 
         */
        public function leggiSingolaRiga($tabella,$valoreDaCercare,$colonnaDaCercare = "id") {
            if(is_numeric($valoreDaCercare)) {
                $query = "SELECT * FROM $tabella  WHERE $colonnaDaCercare = $valoreDaCercare";
            }
            else {
                $query = "SELECT * FROM $tabella  WHERE $colonnaDaCercare LIKE '$valoreDaCercare'";
            }
            $risQuery = $this->eseguiQueryLettura($query);
            if($risQuery->num_rows>0) {                    
                return $risQuery->fetch_assoc();
            }
            else {
                return false;
            }
        }
        
        /*
         * restituisce l'ultima riga della tabella selezionata, restituendo l'array associativo contenente 
         * tutti i suoi elementi, false se la tabella è vuota
         * 
         * @param $tabella la tabella sulla quale eseguire la query
         * @param $colonnaOrdinamento contiene il nome della colonna con cui effettuare l'ordinamento. È un campo opzionale,
         * se non viene inserito usa il valore di default che è id
         * 
         */
        public function ultimaRigaTabella($tabella,$colonnaOrdinamento = "id") {
           
            $query = "SELECT * FROM $tabella  ORDER  BY $colonnaOrdinamento DESC";
            $risQuery = $this->eseguiQueryLettura($query);
            if($risQuery->num_rows>0) {                    
                return $risQuery->fetch_assoc();
            }
            else {
                return false;
            }
        }
        
        /*
         * metodo per eseguire più query in scrittura, 
         * @param $arrayQuery prende in input un array di query
         */
        public function eseguiInsiemeDiQuery($arrayQuery) {
            foreach($arrayQuery as $query) {
                if(!$this->db->query($query) && $this->debug=="si") {
                    echo "Errore durante l'esecuzione della query: ".$this->db->error;
                    echo "<br>$query";
                    exit();
                }
            }
        }

        /*
         * metodo che restituisce il numero di elementi  della tabella passata come parametro
         * @parametri è  un array associativo che contiene caratteri opzionali
         * , se vengono inseriti il metodo esegue
         * il conteggio 
         */
        public function contaRigheTabella($tabella,$parametri=null) {
            if($parametri==null) {
                $query = "SELECT COUNT(*) FROM $tabella";
            }
            else {
                $query = "SELECT COUNT(*) FROM $tabella WHERE ";
                foreach($parametri as $parametro=>$valore) {
                    if(is_numeric($valore)) {
                        $query .= " $parametro = $valore AND ";
                    }
                    else {
                        $query .= " $parametro LIKE '$valore' AND ";
                    }
                }
                $query = substr($query,0,strlen($query)-5);
            }
            $ris = $this->eseguiQueryLettura($query);
            $arrayConteggio = $ris->fetch_array(MYSQLI_NUM);
            return $arrayConteggio[0];
        }
        
        /*
         * metodo che restituisce la somma dei valori di una colonna della tabella passata come parametro
         * @parametro e @valore sono parametri opzionali, se vengono inseriti il metodo esegue
         * il conteggio limitato ai parametri indicati
         */        
        public function sommaValoriColonna($elemento,$tabella,$parametro="",$valore="") {
            if($parametro=="") {
                $query = "SELECT SUM($elemento) FROM $tabella";
            }
            else {
                if(is_numeric($valore)) {
                    $query = "SELECT SUM($elemento) FROM $tabella WHERE $parametro = $valore";
                }
                else {
                    $query = "SELECT SUM($elemento) FROM $tabella WHERE $parametro LIKE '$valore'";
                }
            }

            $ris = $this->eseguiQueryLettura($query);
            $arrayConteggio = $ris->fetch_array(MYSQLI_NUM);
            return $arrayConteggio[0];
        }
        
        /*
         * Questo metodo esporta
         */
        public function esportaDatabaseSuFile($percorso) {
            $nomeFile = date("d-m-Y")."$this->nome.sql";
            exec("mysqldump --user={$this->utente} --password={$this->password} --host={$this->host} {$this->nome} --result-file='$percorso/$nomeFile' 2>&1", $output);
            print_r($output);
        }
        
        /*
         * metodo che restituisce le tabelle del database in uso
         */
        public function elencoTabelle() {
            $query = "SHOW tables";
            return $this->eseguiQueryLettura($query);
        }

        /*
         * metodo che restituisce le colonne della tabella passata come parametro
         */      
        public function elencoColonneTabella($tabella) {
            $query = "SHOW COLUMNS FROM $tabella";
            return $this->eseguiQueryLettura($query);
        }
        
        /*
         * metodo che consente di ottenere l'id dell'ultimo elemento memorizzato nel db
         */
        public function idUltimoElemento() {
            return $this->db->insert_id;
        }
    }