<?php 
    /**
    *
    * copyright (c) 2015-2019 Heliac Technologies
    * license http://opensource.org/licenses/gpl-license.php GNU GPL v3 
    *
    */

    class ElaboratoreImmagini {
        public $altezza;
        public $larghezza;
        public $tipoImmagine;
        public $immagine;
        public $sorgente;
        
        /**
         * @param type $sorgente 
         * campo in cui viene inserito il sordente dell'immagine, che può essere un path,
         * oppure una stringa o ancora una stringa in base64
         * 
         * @param type $tipo
         * questo parametro deve essere specificato solo se si sta cercando di creare un'immagine
         * da stringa, base64 o url remota
         */
        function __construct($sorgente, $tipo = "file") {
            $this->sorgente = $sorgente;
            if($tipo==="file") {
                $this->tipoImmagine = pathinfo($sorgente, PATHINFO_EXTENSION);
                if($this->tipoImmagine == 'jpeg') $this->tipoImmagine = 'jpg';
                switch($this->tipoImmagine) {
                    case 'gif': $this->immagine = imagecreatefromgif($sorgente); break;
                    case 'jpg': $this->immagine = imagecreatefromjpeg($sorgente); break;
                    case 'png': $this->immagine = imagecreatefrompng($sorgente); break;
                    default : return "Tipo immagine non supportato!";
                }
            }
            else {
                switch($tipo) {
                    case 'stringa': 
                        $this->immagine = imagecreatefromstring($sorgente); 
                        break;
                    case 'base64': 
                        $this->immagine = imagecreatefromstring($sorgente); 
                        break;
                    case 'urlRemota': 
                        $stringaImmagine = file_get_contents($sorgente);
                        $this->immagine = imagecreatefromstring($stringaImmagine); 
                        break;
                    default : return "Tipo immagine non supportato!";
                }
            }
            $this->setAltezza(imagesy($this->immagine));
            $this->setLarghezza(imagesx($this->immagine));
        }

        public function salva($percorso) {
            switch($this->tipoImmagine) {
              case 'gif': imagegif($this->immagine, $percorso); break;
              case 'jpg': imagejpeg($this->immagine, $percorso); break;
              case 'png': imagepng($this->immagine, $percorso); break;
            }
        }

        /*
         * questo metodo consente di ridurre o ingrandire in scala l'immagine presente nel
         * valore $this->immagine. Calcola la scala dell'immagine 
         * in base ai parametri dati, poi calcola le nuove dimensioni e crea
         * la nuova immagine
         * 
         * Prende in input tre campi:
         * @parametro   può essere altezza o larghezza
         * @valore      il valore in pixel del parametro specificato sopra
         * @percorso    il percorso dove salvare l'immagine
         * 
         * Esempio di funzionamento: Immagine base di 1600 * 1200
         * il metodo riceve questi valori:
         * @parametro = larghezza
         * @valore    = 1000
         * @percorso  = un percorso qualsiasi
         * 
         * il metodo calcolerà la scala considerando la larghezza, ovvero:
         * $scala = 1000 / 1600
         * 
         * conseguentemente calcolerà le immagini e creerà la nuova immagine
         */
        public function modificaInScala($parametro,$valore, $percorso) {
            $scala = 1;
            if($parametro=="larghezza") {
                $scala = $valore / $this->getLarghezza();
            }
            else {
                $scala = $valore / $this->getAltezza();
            }
            $larghezzaNuovaImmagine = ceil($this->getLarghezza() * $scala);
            $altezzaNuovaImmagine   = ceil($this->getAltezza() * $scala);
            $thumb = imagecreatetruecolor($larghezzaNuovaImmagine,$altezzaNuovaImmagine);
            imagecopyresampled($thumb, $this->immagine, 0, 0, 0, 0, $larghezzaNuovaImmagine, $altezzaNuovaImmagine, $this->getLarghezza(), $this->getAltezza());
            switch($this->tipoImmagine) {
                case 'gif': 
                    imagegif($thumb, $percorso); 
                    break;
                case 'jpg': 
                    imagejpeg($thumb, $percorso); 
                    break;
                case 'png': 
                    imagepng($thumb, $percorso); 
                    break;
            }
        }

        public function riduciInPercentuale($percentuale,$percorso) {
            $altezza   = round($this->altezza*$percentuale/100);
            $larghezza = round($this->larghezza*$percentuale/100);
            $thumb = imagecreatetruecolor($larghezza,$altezza);
            imagecopyresampled($thumb, $this->immagine, 0, 0, 0, 0, $larghezza, $altezza, $this->getLarghezza(), $this->getAltezza());
            switch($this->tipoImmagine) {
                case 'gif': 
                    imagegif($thumb, $percorso); 
                    break;
                case 'jpg': 
                    imagejpeg($thumb, $percorso); 
                    break;
                case 'png': 
                    imagepng($thumb, $percorso); 
                    break;
            }
        }

        /*
         * metodo per il ritaglio di un'immagine. Prende in input i seguenti parametri
         * 
         * @puntoIniziale: array contenente le coordinate x,y del punto di partenza
         * @larghezza:     
         * @altezza:
         * @destinazione:  percorso di destinazione del file
         */
        public function ritagliaImmagine($puntoIniziale,$larghezza,$altezza,$destinazione) {
            if($this->larghezza < $larghezza or $this->altezza < $altezza) {
                return "L'immagine è troppo piccola!";
                exit();
            }
            $x = $puntoIniziale[0];
            $y = $puntoIniziale[1];
            
            // se è una png o una gif preserva la transparenza
            if($this->tipoImmagine == "gif" || $this->tipoImmagine == "png"){
                imagecolortransparent($immagineDestinazione, imagecolorallocatealpha($new, 0, 0, 0, 127));
                imagealphablending($immagineDestinazione, false);
                imagesavealpha($immagineDestinazione, true);
            }
            $immagineDestinazione = imagecrop($this->immagine, array('x' => $x, 'y' => $y, 'width' => $larghezza, 'height' => $altezza));

            switch($this->tipoImmagine){
                case 'gif': imagegif($immagineDestinazione, $destinazione); break;
                case 'jpg': imagejpeg($immagineDestinazione, $destinazione); break;
                case 'png': imagepng($immagineDestinazione, $destinazione); break;
            }
        }
        
        /*
         * consente di ruotare l'immagine di una data quantità di dati
         * se @param $nonSovrascrivere = false la variabile istanza
         * $this->immagine viene sovrascritta con i dati dell'immagine ruotata, 
         * altrimenti l'immagine ruotata viene restituita
         */
        public function ruotaImmagine($gradi,$nonSovrascrivere=false) {
            if($nonSovrascrivere) {
                return imagerotate($this->immagine, $gradi, 0);
            }
            else {
                $this->immagine = imagerotate($this->immagine, $gradi, 0);
                return true;
            }
        }

        // metodi dedicati alla gestione dei dati exif
        public function datiExif() {
            return exif_read_data($this->sorgente);
        }
        
        /*
         * Questo metodo restituisce solo l'orientamento impsotato nei metadati, dato che si tratta
         * di uno dei dati più richiesti è stato creato un metodo apposta
         */
        public function orientamentoExif() {
            $exif = $this->datiExif();
            return $exif["Orientation"];
        }
        
        // getter & setter
        public function getAspectRatio() {
            return $this->larghezza/$this->altezza;
        }
        
        public function getStringaImmagineBase64() {
            $datiImg = file_get_contents($this->percorsoImmagne);
            return base64_encode($datiImg);
        }

        public function getAltezza() {
            return $this->altezza;
        }

        public function setAltezza($altezza) {
            $this->altezza = $altezza;
        }

        public function getLarghezza() {
            return $this->larghezza;
        }

        public function setLarghezza($larghezza) {
            $this->larghezza = $larghezza;
        }

        public function getTipoImmagine() {
            return $this->tipoImmagine;
        }

        public function setTipoImmagine($tipoImmagine) {
            $this->tipoImmagine = $tipoImmagine;
        }

        public function getPercorsoImmagne() {
            return $this->percorsoImmagne;
        }

        public function setPercorsoImmagne($sorgente) {
            $this->percorsoImmagne = $sorgente;
        }
    }