<?php
    /**
    *
    * copyright (c) 2015-2020 Heliac Technologies
    * license http://opensource.org/licenses/gpl-license.php GNU GPL v3 
    *
    */

    class Ricerca {
        private $db;
        private $tabella;
        private $ordinamento;
        private $query;
        private $elementiPerPagina;
        private $parametriRicerca; //array associativo
        private $paginaCorrente;
        private $risultatoQueryRicerca;
        private $numeroOggettiTrovati;
        private $numeroPagine;
        private $templateStampa;
        private $stringaNessunRisultato;
        private $ricercaTerminiEsatti;

        function __construct($db,$tabella,$elementiPerPagina,$parametriRicerca,$templateStampa,$stringaNessunRisultato,$ricercaTerminiEsatti = false) {
            $this->db = $db;
            $this->tabella = $tabella;
            $this->elementiPerPagina=$elementiPerPagina;
            $this->templateStampa = $templateStampa;
            $this->parametriRicerca = $parametriRicerca;
            $this->stringaNessunRisultato = $stringaNessunRisultato;
            $this->ordinamento = "";
            $this->query = "";
            $this->risultatoQueryRicerca = null;
            $this->numeroOggettiTrovati = 0;
            $this->numeroPagine = 0;
            $this->ricercaTerminiEsatti = $ricercaTerminiEsatti;
        }

        public function eseguiRicerca($pagina,$operatoreLogico = "AND") {
            $this->eseguiQueryConteggio();
            $this->paginaCorrente = $pagina;
            if($this->paginaCorrente==1) {
                $stringaLimiteRisultati = "LIMIT $this->elementiPerPagina";
            }
            else {
                $elementiPrecedenti = ($this->paginaCorrente-1) * $this->elementiPerPagina;
                $stringaLimiteRisultati = "LIMIT $elementiPrecedenti, $this->elementiPerPagina";
            }

            $stringaParametriRicerca = "";
            $i = 1;
            foreach($this->parametriRicerca as $nomeParametro=>$parametro) {
                if(is_numeric($parametro)) { // se il parametro passato in input è un numero usa l'operatore =
                    $stringaParametro = "$nomeParametro =$parametro";
                }
                else { //altrimenti è una stringa e usa LIKE
                    $stringaParametro = ($this->ricercaTerminiEsatti==true) ? "$nomeParametro  LIKE '$parametro' " : "$nomeParametro  LIKE '%$parametro%' ";
                }
                  
                if($i==1) {
                    $stringaParametriRicerca .="WHERE $stringaParametro "; 
                }
                else {
                    $stringaParametriRicerca .="$operatoreLogico $stringaParametro "; 
                }
                $i++;
            }
            $this->query = "SELECT * FROM {$this->tabella} $stringaParametriRicerca";
            if($this->ordinamento!="") {
                 $this->query =  $this->query." ".$this->ordinamento." ".$stringaLimiteRisultati;
            }
            else {
                 $this->query =  $this->query." ".$stringaLimiteRisultati;
            }
            $this->risultatoQueryRicerca = $this->db->eseguiQueryLettura($this->query);
            return $this->risultatoQueryRicerca;
        }

        /*
         * esegue la query per contare il numero totale di oggetti presenti della ricerca con i 
         * parametri inseriti. Il metodo imposta anche il numero di pagine dividendo il numero
         * di risultati trovati per il numero di elementi per pagina
         */
        public function eseguiQueryConteggio() {
            $stringaParametriRicerca = "";
            $i = 1;
            foreach($this->parametriRicerca as $nomeParametro=>$parametro) {
                if(is_numeric($parametro)) { // se il parametro passato in input è un numero usa l'operatore =
                    $stringaParametro = "$nomeParametro =$parametro";
                }
                else { //altrimenti è una stringa e usa LIKE
                    $stringaParametro = ($this->ricercaTerminiEsatti==true) ? "$nomeParametro  LIKE '$parametro' " : "$nomeParametro  LIKE '%$parametro%' ";
                }
                  
                if($i==1) {
                    $stringaParametriRicerca .="WHERE $stringaParametro "; 
                }
                else {
                    $stringaParametriRicerca .="AND $stringaParametro "; 
                }
                $i++;
            }
            $this->query = "SELECT COUNT(*) FROM {$this->tabella} $stringaParametriRicerca";
            $this->risultatoQueryRicerca = $this->db->eseguiQueryLettura($this->query);
            $risultato = $this->risultatoQueryRicerca->fetch_array();
            $this->numeroOggettiTrovati = $risultato[0];
            $this->numeroPagine = ceil($this->numeroOggettiTrovati/$this->elementiPerPagina);
        }
        
        public function getNumeroPagine() {
            return ceil($this->numeroOggettiTrovati / $this->elementiPerPagina);
        }
        
        public function eseguiRicercaCasuale($pagina) {
            $query = "SELECT * FROM $this->tabella  ORDER BY RAND() LIMIT {$this->elementiPerPagina}";
            $this->risultatoQueryRicerca = $this->db->eseguiQueryLettura($query);
            $this->numeroOggettiTrovati = $this->risultatoQueryRicerca->num_rows;
            $this->numeroPagine = round($this->numeroOggettiTrovati/$this->elementiPerPagina);
        }
        
        public function stampaRisultatiRicerca() {
            if($this->risultatoQueryRicerca->num_rows>0) {
                while($oggetto = $this->risultatoQueryRicerca->fetch_assoc()) {   
                    $this->stampaElementoCorrente($oggetto);
                }
            }
            else {
                echo $this->stringaNessunRisultato;
            }
        }

        /*
         *  metodo che prende il template in input e sostituisce le occorrenze __nomecampo__ prendendo
         *  i nomi dei campi dall'array associativo restituito
         */
        public function stampaElementoCorrente($oggetto) {
            $rigaDaStampare = $this->templateStampa;
            foreach($oggetto as $nomeParametro=>$valore) {
                $rigaDaStampare = str_replace("__".$nomeParametro."__", $valore, $rigaDaStampare);
            }
            echo $rigaDaStampare;
        }

        /*
         * stampa la paginazione per la ricerca, ha bisogno di ricevere il link compreso
         * di tutti i parametri tranne ovviamente il numero di pagina
         */
        public function stampaPaginazione($linkPagina) {
            if($this->numeroOggettiTrovati > $this->elementiPerPagina) {
                echo"<ul class=\"pagination pagination-sm\">";
                $numeroPagine = ceil($this->numeroOggettiTrovati / $this->elementiPerPagina);
                if($this->paginaCorrente==1) {
                    echo"<li class=\"disabled\"><a href=\"#\">&laquo;</a>";
                }
                else {
                    $prec = $this->paginaCorrente-1;
                    echo"<li ><a href=\"".$linkPagina.$prec.">&laquo;</a>";
                }
                for($i=1;$i<$numeroPagine+1;$i++) {
                    if($i==$this->paginaCorrente) {
                        echo"<li class=\"active\"><a href=\"$linkPagina"."$i\">$i</a></li>";				
                    }
                    else {
                        echo"<li><a href=\"$linkPagina"."$i\">$i</a></li>";
                    }
                }
                if($this->paginaCorrente==$numeroPagine) {
                    echo"<li class=\"disabled\"><a href=\"\">&raquo;</a>";
                }
                else {
                    $succ = $this->paginaCorrente+1;
                    echo"<li ><a href=\"".$linkPagina.$succ."\">&raquo;</a>";
                }
                echo"</ul>";
            }
        }

        //gettere e setter
        function getTabella() {
            return $this->tabella;
        }

        function getQuery() {
            return $this->query;
        }

        function getElementiPerPagina() {
            return $this->elementiPerPagina;
        }

        function getParametriRicerca() {
            return $this->parametriRicerca;
        }

        function getRisultatoQueryRicerca() {
            return $this->risultatoQueryRicerca;
        }

        function getNumeroOggettiTrovati() {
            return $this->numeroOggettiTrovati;
        }

        function getTemplateStampa() {
            return $this->templateStampa;
        }

        function getStringaNessunRisultato() {
            return $this->stringaNessunRisultato;
        }

        function getRicercaTerminiEsatti() {
            return $this->ricercaTerminiEsatti;
        }

        function setRicercaTerminiEsatti($ricercaTerminiEsatti) {
            $this->ricercaTerminiEsatti = $ricercaTerminiEsatti;
        }

        function setTabella($tabella) {
            $this->tabella = $tabella;
        }

        function setQuery($query) {
            $this->query = $query;
        }

        function getOrdinamento() {
            return $this->ordinamento;
        }

        function getPaginaCorrente() {
            return $this->paginaCorrente;
        }

        function setOrdinamento($ordinamento) {
            $this->ordinamento = $ordinamento;
        }

        function setPaginaCorrente($paginaCorrente) {
            $this->paginaCorrente = $paginaCorrente;
        }

        function setElementiPerPagina($elementiPerPagina) {
            $this->elementiPerPagina = $elementiPerPagina;
        }

        function setParametriRicerca($parametriRicerca) {
            $this->parametriRicerca = $parametriRicerca;
        }

        function setRisultatoQueryRicerca($risultatoQueryRicerca) {
            $this->risultatoQueryRicerca = $risultatoQueryRicerca;
        }

        function setNumeroOggettiTrovati($numeroOggettiTrovati) {
            $this->numeroOggettiTrovati = $numeroOggettiTrovati;
        }

        function setTemplateStampa($templateStampa) {
            $this->templateStampa = $templateStampa;
        }

        function setStringaNessunRisultato($stringaNessunRisultato) {
            $this->stringaNessunRisultato = $stringaNessunRisultato;
        }
    }